@@ -38,6 +38,7 @@ struct _NautilusFileConflictDialog
     GtkDialog parent_instance;
 
     gchar *conflict_name;
+    gchar *suggested_name;
 
     /* UI objects */
     GtkWidget *titles_vbox;
@@ -55,6 +56,8 @@ struct _NautilusFileConflictDialog
 
 G_DEFINE_TYPE (NautilusFileConflictDialog, nautilus_file_conflict_dialog, GTK_TYPE_DIALOG);
 
+#define MAX_LABEL_WIDTH 50
+
 void
 nautilus_file_conflict_dialog_set_text (NautilusFileConflictDialog *fcd,
                                         gchar                      *primary_text,
@@ -67,6 +70,7 @@ nautilus_file_conflict_dialog_set_text (NautilusFileConflictDialog *fcd,
     gtk_label_set_line_wrap (GTK_LABEL (label), TRUE);
     gtk_label_set_line_wrap_mode (GTK_LABEL (label), PANGO_WRAP_WORD_CHAR);
     gtk_label_set_xalign (GTK_LABEL (label), 0.0);
+    gtk_label_set_max_width_chars (GTK_LABEL (label), MAX_LABEL_WIDTH);
     gtk_box_pack_start (GTK_BOX (fcd->titles_vbox), label, FALSE, FALSE, 0);
     gtk_widget_show (label);
 
@@ -79,7 +83,9 @@ nautilus_file_conflict_dialog_set_text (NautilusFileConflictDialog *fcd,
 
     label = gtk_label_new (secondary_text);
     gtk_label_set_line_wrap (GTK_LABEL (label), TRUE);
+    gtk_label_set_line_wrap_mode (GTK_LABEL (label), PANGO_WRAP_WORD_CHAR);
     gtk_label_set_xalign (GTK_LABEL (label), 0.0);
+    gtk_label_set_max_width_chars (GTK_LABEL (label), MAX_LABEL_WIDTH);
     gtk_box_pack_start (GTK_BOX (fcd->titles_vbox), label, FALSE, FALSE, 0);
     gtk_widget_show (label);
 }
@@ -135,8 +141,14 @@ nautilus_file_conflict_dialog_set_conflict_name (NautilusFileConflictDialog *fcd
                                                  gchar                      *conflict_name)
 {
     fcd->conflict_name = g_strdup (conflict_name);
+}
 
-    gtk_entry_set_text (GTK_ENTRY (fcd->entry), fcd->conflict_name);
+void
+nautilus_file_conflict_dialog_set_suggested_name (NautilusFileConflictDialog *fcd,
+                                                  gchar                      *suggested_name)
+{
+    fcd->suggested_name = g_strdup (suggested_name);
+    gtk_entry_set_text (GTK_ENTRY (fcd->entry), suggested_name);
 }
 
 void
@@ -168,45 +180,58 @@ static void
 entry_text_changed_cb (GtkEditable                *entry,
                        NautilusFileConflictDialog *dialog)
 {
-    /* The rename button is visible only if there's text
-     * in the entry.
-     */
     if  (g_strcmp0 (gtk_entry_get_text (GTK_ENTRY (entry)), "") != 0 &&
          g_strcmp0 (gtk_entry_get_text (GTK_ENTRY (entry)), dialog->conflict_name) != 0)
+    {
+        gtk_widget_set_sensitive (dialog->rename_button, TRUE);
+    }
+    else
+    {
+        gtk_widget_set_sensitive (dialog->rename_button, FALSE);
+    }
+}
+
+static void
+on_expanded_notify (GtkExpander                *w,
+                    GParamSpec                 *pspec,
+                    NautilusFileConflictDialog *dialog)
+{
+    if (gtk_expander_get_expanded (w))
     {
         gtk_widget_hide (dialog->replace_button);
         gtk_widget_show (dialog->rename_button);
+        gtk_dialog_set_default_response (GTK_DIALOG (dialog), CONFLICT_RESPONSE_RENAME);
 
         gtk_widget_set_sensitive (dialog->checkbox, FALSE);
 
-        gtk_dialog_set_default_response (GTK_DIALOG (dialog), CONFLICT_RESPONSE_RENAME);
+        gtk_widget_grab_focus (dialog->entry);
+        if (g_strcmp0 (gtk_entry_get_text (GTK_ENTRY (dialog->entry)), dialog->suggested_name) == 0)
+        {
+            /* The suggested name is in the form "original (1).txt", if the
+             * the conflicting name was "original.txt". The user may want to
+             * replace the "(1)" bits with with something more meaningful, so
+             * select this region for convenience. */
+
+            const gchar *offset;
+            int start_pos;
+            int end_pos;
+
+            offset = eel_filename_get_extension_offset (dialog->conflict_name);
+            start_pos = g_utf8_pointer_to_offset (dialog->conflict_name, offset);
+
+            offset = eel_filename_get_extension_offset (dialog->suggested_name);
+            end_pos = g_utf8_pointer_to_offset (dialog->suggested_name, offset);
+
+            gtk_editable_select_region (GTK_EDITABLE (dialog->entry), start_pos, end_pos);
+        }
     }
     else
     {
         gtk_widget_hide (dialog->rename_button);
         gtk_widget_show (dialog->replace_button);
+        gtk_dialog_set_default_response (GTK_DIALOG (dialog), CONFLICT_RESPONSE_REPLACE);
 
         gtk_widget_set_sensitive (dialog->checkbox, TRUE);
-
-        gtk_dialog_set_default_response (GTK_DIALOG (dialog), CONFLICT_RESPONSE_REPLACE);
-    }
-}
-
-static void
-expander_activated_cb (GtkExpander                *w,
-                       NautilusFileConflictDialog *dialog)
-{
-    int start_pos, end_pos;
-
-    if (!gtk_expander_get_expanded (w))
-    {
-        if (g_strcmp0 (gtk_entry_get_text (GTK_ENTRY (dialog->entry)), dialog->conflict_name) == 0)
-        {
-            gtk_widget_grab_focus (dialog->entry);
-
-            eel_filename_get_rename_region (dialog->conflict_name, &start_pos, &end_pos);
-            gtk_editable_select_region (GTK_EDITABLE (dialog->entry), start_pos, end_pos);
-        }
     }
 }
 
@@ -215,11 +240,8 @@ checkbox_toggled_cb (GtkToggleButton            *t,
                      NautilusFileConflictDialog *dialog)
 {
     gtk_widget_set_sensitive (dialog->expander, !gtk_toggle_button_get_active (t));
-    gtk_widget_set_sensitive (dialog->rename_button, !gtk_toggle_button_get_active (t));
 
-    if  (!gtk_toggle_button_get_active (t) &&
-         g_strcmp0 (gtk_entry_get_text (GTK_ENTRY (dialog->entry)), "") != 0 &&
-         g_strcmp0 (gtk_entry_get_text (GTK_ENTRY (dialog->entry)), dialog->conflict_name) != 0)
+    if  (!gtk_toggle_button_get_active (t))
     {
         gtk_widget_hide (dialog->replace_button);
         gtk_widget_show (dialog->rename_button);
@@ -290,8 +312,8 @@ nautilus_file_conflict_dialog_init (NautilusFileConflictDialog *fcd)
     /* Setup the expander for the rename action */
     fcd->expander = gtk_expander_new_with_mnemonic (_("_Select a new name for the destination"));
     gtk_box_pack_start (GTK_BOX (vbox2), fcd->expander, FALSE, FALSE, 0);
-    g_signal_connect (fcd->expander, "activate",
-                      G_CALLBACK (expander_activated_cb), dialog);
+    g_signal_connect (fcd->expander, "notify::expanded",
+                      G_CALLBACK (on_expanded_notify), dialog);
 
     hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 6);
     gtk_container_add (GTK_CONTAINER (fcd->expander), hbox);
@@ -332,6 +354,7 @@ nautilus_file_conflict_dialog_init (NautilusFileConflictDialog *fcd)
                                                 _("Re_name"),
                                                 CONFLICT_RESPONSE_RENAME);
     gtk_widget_hide (fcd->rename_button);
+    gtk_widget_set_no_show_all (fcd->rename_button, TRUE);
 
     fcd->replace_button = gtk_dialog_add_button (dialog,
                                                  _("Re_place"),
@@ -352,6 +375,7 @@ do_finalize (GObject *self)
     NautilusFileConflictDialog *dialog = NAUTILUS_FILE_CONFLICT_DIALOG (self);
 
     g_free (dialog->conflict_name);
+    g_free (dialog->suggested_name);
 
     G_OBJECT_CLASS (nautilus_file_conflict_dialog_parent_class)->finalize (self);
 }
